/* msine.c
	Copyright 1998, 2001  Jed Margolin
	Lissajous Figures
*/

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <process.h>
#include <graphics.h>
#include <math.h>

#define RETURN 13
#define UPARROW 72
#define DOWNARROW 80
#define LEFTARROW 75
#define RIGHTARROW 77
#define END 79
#define PgUp 73
#define PgDn 81
#define CTL_PgUp 132
#define CTL_PgDn 118
#define ESC 27
#define FF 13
#define CR 10
#define TRUE 1

extern int vsync_flag;

void sine1(void);
void wait_for_retrace(void);
void sine_program(int,int),rsine_program(int,int);
void waveb1(int);
void waveb2(int);

#define DEG 512			// circle divider, binary multiples only
#define XSCALE 256*1.3		// x screen scaling
#define YSCALE 350*1.3		// y screen scaling
#define XOFFSET 320	   	// screen x offset
#define YOFFSET 180	   	// screen y offset
#define SPEED 4         	// adjust control sensitivity

int sin_table[DEG];
int scolor,bkcolor;
int wsin(int);

void sine1()
{
	int driver, mode, n, page, dquit, mscale, bscale;
	double rads;
	unsigned char c;

	bkcolor = BLACK;
	scolor = GREEN;

// Initialize Graphics
	driver = VGA;
	mode = VGAMED;
	registerbgidriver(EGAVGA_driver);
	initgraph(&driver, &mode, "");
	setbkcolor(bkcolor);
	setcolor(scolor);
//-------------------------------------------------------------------------
// Make the sine table
	rads = 3.14159/180.0*360.0/DEG;
	for(n=0;n<DEG;n++)
	{
		sin_table[n] =  ( sin(n*rads) * 32000 );
	}

	page = 0;
	mscale = 3;
	bscale = 1;
// ---------------------------------------------------------------------
// Read and process keyboard
	dquit = 0;

	while(!(dquit))
	{
		if(page == 0)	page = 1;	// video page ping-pong
		else		page = 0;

		setactivepage(page);  // directs the output of the graphics functions
		cleardevice();        // clears the screen (visual page)
		setcolor(scolor);

		if(bscale>0)	sine_program(mscale,bscale);	// sine program
		else		rsine_program(mscale,bscale);			// rsine program

		setvisualpage(page);			// displays the video page
		wait_for_retrace();
		if( kbhit() )
		{
		  c = getch() ;
		  if (!c) c = getch() ;		// get second byte if cursor key
		  switch (c)
		  {
			  case RIGHTARROW :
				 mscale++;
				 break ;

			  case LEFTARROW :
				 mscale--;
				 break ;

			  case UPARROW :
				 bscale++;
				 if(bscale==0)   bscale = 2;
				 break ;

			  case DOWNARROW :
				 bscale--;
				 if(bscale==0)   bscale = -2;
				 break ;

			  case PgDn :
				 scolor--;
				 scolor = scolor&0x0F;
				 break ;

			  case PgUp :
				 scolor++;
				 scolor = scolor&0x0F;
				 break ;

			  case CTL_PgDn :
				 bkcolor--;
				 bkcolor = bkcolor&0x0F;
				 setbkcolor(bkcolor);
				 break ;

			  case CTL_PgUp :
				 bkcolor++;
				 bkcolor = bkcolor&0x0F;
				 setbkcolor(bkcolor);
				 break ;

			  case RETURN :
				 dquit = TRUE;
				 break ;

			  case ESC :
				 dquit = TRUE;
				 break ;
		  }				// end of switch
		}	  			// end if
	}		  		// end while

	restorecrtmode();
}
// ---------------------------------------------------------------------
// Wrap sine table index and look it up
int wsin(n)
int n;
{
	n = n & (DEG-1);
	return(sin_table[n]);
}
// ---------------------------------------------------------------------
void sine_program(int mscale, int bscale)
{
	int i,newx,newy,oldx,oldy;
	static int m;

	oldx = wsin(0)/XSCALE + XOFFSET;					// sx
	oldy = wsin(m/SPEED)/YSCALE + YOFFSET;					// sy

	for(i=0;i<(DEG+1);i++)
	{
		newx = wsin(i)/XSCALE + XOFFSET;						// sx
		newy = wsin(bscale*i+m/SPEED)/YSCALE + YOFFSET;	// sy
		line(oldx, oldy, newx, newy);

		oldx = newx;
		oldy = newy;
	}

	m = m + mscale;
	m = m & (SPEED*DEG-1);

	}
// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
void rsine_program(int mscale, int bscale)
{
	int i,newx,newy,oldx,oldy;
	static int m;

	oldx = wsin(m/SPEED)/XSCALE + XOFFSET;			// sx
	oldy = wsin(0)/YSCALE + YOFFSET;					// sy

	for(i=0;i<(DEG+1);i++)
	{
		newx = wsin(-bscale*i+m/SPEED)/XSCALE + XOFFSET;	// sx
		newy = wsin(i)/YSCALE + YOFFSET;							// sy
		line(oldx, oldy, newx, newy);

		oldx = newx;
		oldy = newy;
	}

	m = m + mscale;
	m = m & (SPEED*DEG-1);
}
// ---------------------------------------------------------------------
// Wait for Vertical Retrace
void wait_for_retrace()
{

// spin while vsync is low (wait for it to go high)
	while( (inp(0x3DA) & 8) == 0 ) ;   

// spin while vsync is high (wait for it to go low)
	while( (inp(0x3DA) & 8) == 8 ) ;   

		
/* This doesn't work properly on my ABIT motherboard with an AGP graphics
   board. (ATI Xpert 99)

   If you really need to run this program on such a machine, you may 
   have to use a software timing loop such as:

	void wait_for_retrace()
	{
	   long n;

	   for(n=0;n<1000000L; n++);
	}

   This makes it non-portable; perhaps you can figure out a better
   solution. (Why are you running a DOS program on a machine with an
   AGP graphics board, anyway?)
*/

}
// ---------------------------------------------------------------------

